@@ -1,3 +1,4 @@
+
 ###{{{ print.twinlm
 
 ##' @S3method print twinlm
@@ -45,8 +46,6 @@ summary.twinlm <- function(object,...) {
     return(res)
   }
 
-  ##  suppressMessages(browser())
-
   OScor <- NULL
   if(object$OS) {
     OScor <- constraints(e,k=3)[,c(1,5,6),drop=FALSE]
@@ -284,3 +283,5 @@ acde.twinlm <- function(x,...) {
 }
 
 ###}}} acde
+
+coef.twinlm <- function(object,...) coef(object$estimate,...)
