@@ -55,10 +55,11 @@ twinsim <- function(nMZ=100,nDZ=nMZ,b1=c(),b2=c(),mu=0,acde=c(1,1,0,1),randomslo
   if (length(b2)>0) { d <- cbind(d,g=g) }
   colnames(d) <- sub(".","",colnames(d),fixed=TRUE)
   if (wide) return(d)
-  dd <- reshape(d,direction="long",varying=vary)
-  dd$y <- (dd$y1>threshold & dd$y1<cens)*1
-  dd$y0 <- (dd$y1>threshold)
-  dd$status <- dd$y1<dd$cens
+  dd <- fast.reshape(d,idvar="id",varying="y")
+
+  dd$status <- dd$y<dd$cens
+  dd$y0 <- (dd$y>threshold)
+  dd$y1 <- (dd$y>threshold & dd$y<cens)*1
   ## S.MZ <- diag(2)*vE+vC+vA
   ## S.DZ <- diag(2)*(vE+vA) + rho*S2*vA + vC
   ## Mu <- (threshold-mu)
